# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
"""
Common test functions testing launchers
"""

import pytest
from omegaconf import OmegaConf

from hydra.test_utils.test_utils import verify_dir_outputs


class LauncherTestSuite:
    def test_sweep_1_job(self, sweep_runner, launcher_name, overrides):  # noqa: F811
        sweep_1_job(
            sweep_runner, overrides=["hydra/launcher=" + launcher_name] + overrides
        )

    def test_sweep_2_jobs(self, sweep_runner, launcher_name, overrides):  # noqa: F811
        sweep_2_jobs(
            sweep_runner, overrides=["hydra/launcher=" + launcher_name] + overrides
        )

    def test_not_sweeping_hydra_overrides(
        self, sweep_runner, launcher_name, overrides
    ):  # noqa: F811
        not_sweeping_hydra_overrides(
            sweep_runner, overrides=["hydra/launcher=" + launcher_name] + overrides
        )

    def test_sweep_1_job_strict(
        self, sweep_runner, launcher_name, overrides
    ):  # noqa: F811
        sweep_1_job(
            sweep_runner,
            strict=True,
            overrides=["hydra/launcher=" + launcher_name] + overrides,
        )

    def test_sweep_1_job_strict_and_bad_key(
        self, sweep_runner, launcher_name, overrides
    ):  # noqa: F811
        with pytest.raises(KeyError):
            sweep_1_job(
                sweep_runner,
                strict=True,
                overrides=["hydra/launcher=" + launcher_name, "boo=bar"] + overrides,
            )

    def test_sweep_2_optimizers(self, sweep_runner, launcher_name, overrides):
        sweep_two_config_groups(
            sweep_runner, overrides=["hydra/launcher=" + launcher_name] + overrides
        )


def sweep_1_job(sweep_runner, overrides, strict=False):
    """
    Runs a sweep with one job
    """
    sweep = sweep_runner(
        calling_file=None,
        calling_module="hydra.test_utils.a_module",
        config_path="configs/compose.yaml",
        overrides=overrides,
        strict=strict,
    )
    with sweep:
        job_ret = sweep.returns[0]
        assert len(job_ret) == 1
        assert job_ret[0].overrides == []
        assert job_ret[0].cfg == {"foo": 10, "bar": 100}
        assert job_ret[0].hydra_cfg.hydra.job.name == "a_module"
        verify_dir_outputs(sweep.returns[0][0])


def sweep_2_jobs(sweep_runner, overrides):
    """
    Runs a sweep with two jobs
    """
    overrides.append("a=0,1")
    sweep = sweep_runner(
        calling_file=None,
        calling_module="hydra.test_utils.a_module",
        config_path="configs/compose.yaml",
        overrides=overrides,
    )
    base = OmegaConf.create({"foo": 10, "bar": 100, "a": 0})

    with sweep:
        assert len(sweep.returns[0]) == 2
        for i in range(2):
            job_ret = sweep.returns[0][i]
            expected_conf = OmegaConf.merge(
                base, OmegaConf.from_dotlist(job_ret.overrides)
            )
            assert job_ret.overrides == ["a={}".format(i)]
            assert job_ret.cfg == expected_conf
            assert job_ret.hydra_cfg.hydra.job.name == "a_module"
            verify_dir_outputs(job_ret, job_ret.overrides)


def not_sweeping_hydra_overrides(sweep_runner, overrides):
    """
    Runs a sweep with two jobs
    """
    overrides.extend(["a=0,1", "hydra.foo=1,2,3"])
    sweep = sweep_runner(
        calling_file=None,
        calling_module="hydra.test_utils.a_module",
        config_path="configs/compose.yaml",
        overrides=overrides,
    )
    base = OmegaConf.create({"foo": 10, "bar": 100})

    with sweep:
        assert len(sweep.returns[0]) == 2
        for i in range(2):
            job_ret = sweep.returns[0][i]
            expected_conf = OmegaConf.merge(
                base, OmegaConf.from_dotlist(job_ret.overrides)
            )
            assert job_ret.overrides == ["a={}".format(i)]
            assert job_ret.cfg == expected_conf
            verify_dir_outputs(job_ret, job_ret.overrides)


def sweep_two_config_groups(sweep_runner, overrides):
    """
    Make sure that optimizers=adam,nesterov is interpreted correctly
    """
    overrides.extend(["group1=file1,file2"])
    sweep = sweep_runner(
        calling_file=None,
        calling_module="hydra.test_utils.a_module",
        config_path="configs/compose.yaml",
        overrides=overrides,
    )
    expected_overrides = [["group1=file1"], ["group1=file2"]]
    expected_conf = [
        OmegaConf.create({"foo": 10, "bar": 100}),
        OmegaConf.create({"foo": 20, "bar": 100}),
    ]
    with sweep:
        assert len(sweep.returns[0]) == 2
        for i in range(2):
            job_ret = sweep.returns[0][i]
            assert job_ret.overrides == expected_overrides[i]
            assert job_ret.cfg == expected_conf[i]
            verify_dir_outputs(job_ret, job_ret.overrides)



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

import copy
import logging
import os
import re
import sys
from time import strftime, localtime

import six
from omegaconf import OmegaConf, DictConfig

# pylint: disable=C0103
log = logging.getLogger(__name__)


def configure_log(log_config, verbose=None):
    if log_config is not None:
        conf = log_config.to_container(resolve=True)
        logging.config.dictConfig(conf)
    else:
        # default logging to stdout
        root = logging.getLogger()
        root.setLevel(logging.INFO)
        handler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter(
            "[%(asctime)s][%(name)s][%(levelname)s] - %(message)s"
        )
        handler.setFormatter(formatter)
        root.addHandler(handler)

    if verbose is not None:
        if verbose == "root":
            logging.getLogger().setLevel(logging.DEBUG)
        for logger in verbose.split(","):
            logging.getLogger(logger).setLevel(logging.DEBUG)


def save_config(cfg, filename):
    with open(os.path.join(filename), "w") as file:
        file.write(cfg.pretty())


def get_overrides_dirname(lst, exclude_keys=[]):
    lst = [x for x in lst if x not in exclude_keys]
    lst.sort()
    return re.sub(pattern="[=]", repl="=", string=",".join(lst))


def filter_overrides(overrides):
    """
    :param overrides: overrides list
    :return: returning a new overrides list with all the keys starting with hydra. fitlered.
    """
    return [x for x in overrides if not x.startswith("hydra.")]


def run_job(config, task_function, verbose, job_dir_key, job_subdir_key):
    old_cwd = os.getcwd()
    working_dir = str(config.select(job_dir_key))
    if job_subdir_key is not None:
        # evaluate job_subdir_key lazily.
        # this is running on the client side in sweep and contains things such as job:id which
        # are only available there.
        subdir = str(config.select(job_subdir_key))
        working_dir = os.path.join(working_dir, subdir)
    try:
        ret = JobReturn()
        ret.working_dir = working_dir
        task_cfg = copy.deepcopy(config)
        hydra_cfg = task_cfg["hydra"]
        del task_cfg["hydra"]
        ret.cfg = task_cfg
        ret.hydra_cfg = copy.deepcopy(HydraConfig())
        ret.overrides = config.hydra.overrides.task.to_container()
        if not os.path.exists(working_dir):
            os.makedirs(working_dir)
        os.chdir(working_dir)
        configure_log(config.hydra.job_logging, verbose)

        save_config(task_cfg, "config.yaml")
        save_config(hydra_cfg, "hydra.yaml")
        save_config(config.hydra.overrides.task, "overrides.yaml")
        ret.return_value = task_function(task_cfg)
        ret.task_name = JobRuntime().get("name")
        return ret
    finally:
        os.chdir(old_cwd)


def get_valid_filename(s):
    s = str(s).strip().replace(" ", "_")
    return re.sub(r"(?u)[^-\w.]", "", s)


def setup_globals():
    try:
        OmegaConf.register_resolver(
            "now", lambda pattern: strftime(pattern, localtime())
        )

        def job_error(x):
            raise Exception(
                "job:{} is no longer available. use hydra.job.{}".format(x, x)
            )

        OmegaConf.register_resolver("job", job_error)

    except AssertionError:
        # calling it again in no_workers mode will throw. safe to ignore.
        pass


class JobReturn:
    def __init__(self):
        self.overrides = None
        self.return_value = None
        self.cfg = None
        self.hydra_cfg = None
        self.working_dir = None
        self.task_name = None


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

    @staticmethod
    def get_state():
        return Singleton._instances

    @staticmethod
    def set_state(instances):
        Singleton._instances = instances


@six.add_metaclass(Singleton)
class JobRuntime:
    def __init__(self):
        self.conf = OmegaConf.create()
        self.set("name", "UNKNOWN_NAME")

    def get(self, key):
        ret = self.conf.select(key)
        if ret is None:
            raise KeyError("Key not found in {}: {}".format(type(self).__name__, key))
        return ret

    def set(self, key, value):
        log.debug("Setting {}:{}={}".format(type(self).__name__, key, value))
        self.conf[key] = value


@six.add_metaclass(Singleton)
class HydraConfig(DictConfig):
    def __init__(self):
        super(HydraConfig, self).__init__(content={})
        self.hydra = None

    def set_config(self, cfg):
        try:
            OmegaConf.set_readonly(self, False)
            self.hydra = copy.deepcopy(cfg.hydra)
        finally:
            OmegaConf.set_readonly(self, True)



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
import re
import os
import subprocess
import sys

import pytest
from omegaconf import OmegaConf

from hydra.errors import MissingConfigException
from hydra.test_utils.launcher_common_tests import (
    sweep_1_job,
    sweep_2_jobs,
    not_sweeping_hydra_overrides,
    sweep_two_config_groups,
)
from hydra.test_utils.test_utils import chdir_hydra_root, verify_dir_outputs

# noinspection PyUnresolvedReferences
from hydra.test_utils.test_utils import task_runner, sweep_runner  # noqa: F401

chdir_hydra_root()


@pytest.mark.parametrize("calling_file, calling_module", [(".", None), (None, ".")])
def test_missing_conf_dir(task_runner, calling_file, calling_module):  # noqa: F811
    with pytest.raises(MissingConfigException):
        with task_runner(
            calling_file=calling_file,
            calling_module=calling_module,
            config_path="dir_not_found",
        ):
            pass


@pytest.mark.parametrize(
    "calling_file, calling_module", [("hydra/app.py", None), (None, "hydra.app")]
)
def test_missing_conf_file(task_runner, calling_file, calling_module):  # noqa: F811
    with pytest.raises(MissingConfigException):
        with task_runner(
            calling_file=calling_file,
            calling_module=calling_module,
            config_path="not_found.yaml",
        ):
            pass


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app.py", None), (None, "tests.test_app.app")],
)
def test_app_without_config___no_overrides(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file, calling_module=calling_module, config_path=""
    ) as task:
        assert task.job_ret.cfg == {}


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app.py", None), (None, "tests.test_app.app")],
)
def test_app_without_config__with_overrides(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="",
        overrides=["abc=123", "a.b=1", "a.a=2"],
    ) as task:
        assert task.job_ret.cfg == dict(abc=123, a=dict(b=1, a=2))
        verify_dir_outputs(task.job_ret, task.overrides)


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_config_file__no_overrides(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="config.yaml",
    ) as task:
        assert task.job_ret.cfg == dict(
            dataset=dict(name="imagenet", path="/datasets/imagenet")
        )

        verify_dir_outputs(task.job_ret)


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_config_file__with_overide(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="config.yaml",
        overrides=["dataset.path=/datasets/imagenet2"],
    ) as task:
        assert task.job_ret.cfg == dict(
            dataset=dict(name="imagenet", path="/datasets/imagenet2")
        )
        verify_dir_outputs(task.job_ret, task.overrides)


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_split_config(task_runner, calling_file, calling_module):  # noqa: F811
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="split_cfg/config.yaml",
    ) as task:
        assert task.job_ret.cfg == dict(
            dataset=dict(name="imagenet", path="/datasets/imagenet"),
            optimizer=dict(lr=0.001, type="nesterov"),
        )
        verify_dir_outputs(task.job_ret)


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_config_groups__override_dataset__wrong(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with pytest.raises(MissingConfigException) as ex:
        with task_runner(
            calling_file=calling_file,
            calling_module=calling_module,
            config_path="cfg_groups",
            overrides=["optimizer=wrong_name"],
        ):
            pass
    assert sorted(ex.value.options) == sorted(["adam", "nesterov"])


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_config_groups__override_all_configs(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="cfg_groups",
        overrides=["optimizer=adam", "optimizer.lr=10"],
    ) as task:
        assert task.job_ret.cfg == dict(optimizer=dict(type="adam", lr=10, beta=0.01))
        verify_dir_outputs(task.job_ret, overrides=task.overrides)


@pytest.mark.parametrize(
    "calling_file, calling_module",
    [("tests/test_app/app_with_cfg.py", None), (None, "tests.test_app.app_with_cfg")],
)
def test_app_with_sweep_cfg__override_to_basic_launcher(
    task_runner, calling_file, calling_module  # noqa: F811
):
    with task_runner(
        calling_file=calling_file,
        calling_module=calling_module,
        config_path="sweep_cfg/config.yaml",
        overrides=["hydra/launcher=basic"],
    ) as task:
        hydra_cfg = task.job_ret.hydra_cfg
        assert (
            hydra_cfg.hydra.launcher["class"]
            == "hydra._internal.core_plugins.basic_launcher.BasicLauncher"
        )
        assert task.job_ret.hydra_cfg.hydra.launcher.params or {} == {}


@pytest.mark.parametrize(
    "args,output_conf",
    [
        ([], OmegaConf.create()),
        (
            ["abc=123", "hello.a=456", "hello.b=5671"],
            OmegaConf.create(dict(abc=123, hello=dict(a=456, b=5671))),
        ),
    ],
)
def test_demo_0_minimal(tmpdir, args, output_conf):
    cmd = [sys.executable, "demos/0_minimal/minimal.py", "hydra.run.dir=" + str(tmpdir)]
    cmd.extend(args)
    result = subprocess.check_output(cmd)
    assert OmegaConf.create(str(result.decode("utf-8"))) == output_conf


def test_demo_1_workdir(tmpdir):
    cmd = [
        sys.executable,
        "demos/1_working_directory/working_directory.py",
        "hydra.run.dir=" + str(tmpdir),
    ]
    result = subprocess.check_output(cmd)
    assert result.decode("utf-8").rstrip() == "Working directory : {}".format(tmpdir)


@pytest.mark.parametrize(
    "args,expected",
    [
        ([], ["Info level message"]),
        (["-v" "__main__"], ["Info level message", "Debug level message"]),
    ],
)
def test_demo_2_logging(tmpdir, args, expected):
    cmd = [
        sys.executable,
        "demos/2_logging/logging_example.py",
        "hydra.run.dir=" + str(tmpdir),
    ]
    cmd.extend(args)
    result = subprocess.check_output(cmd)
    lines = result.decode("utf-8").splitlines()
    assert len(lines) == len(expected)
    for i in range(len(lines)):
        assert re.findall(expected[i], lines[i])


@pytest.mark.parametrize(
    "args,output_conf",
    [
        (
            [],
            OmegaConf.create(
                dict(dataset=dict(name="imagenet", path="/datasets/imagenet"))
            ),
        ),
        (
            ["dataset.path=abc"],
            OmegaConf.create(dict(dataset=dict(name="imagenet", path="abc"))),
        ),
    ],
)
def test_demo_3_config_file(tmpdir, args, output_conf):
    cmd = [
        sys.executable,
        "demos/3_config_file/config_file.py",
        "hydra.run.dir=" + str(tmpdir),
    ]
    cmd.extend(args)
    result = subprocess.check_output(cmd)
    assert OmegaConf.create(str(result.decode("utf-8"))) == output_conf


@pytest.mark.parametrize(
    "args,output_conf",
    [
        (
            [],
            OmegaConf.create(
                {
                    "dataset": {"name": "imagenet", "path": "/datasets/imagenet"},
                    "optimizer": {"lr": 0.001, "type": "nesterov"},
                }
            ),
        ),
        (
            ["dataset.path=foo"],
            OmegaConf.create(
                {
                    "dataset": {"name": "imagenet", "path": "foo"},
                    "optimizer": {"lr": 0.001, "type": "nesterov"},
                }
            ),
        ),
    ],
)
def test_demo_4_config_splitting(tmpdir, args, output_conf):
    cmd = [
        sys.executable,
        "demos/4_config_splitting/experiment.py",
        "hydra.run.dir=" + str(tmpdir),
    ]
    cmd.extend(args)
    result = subprocess.check_output(cmd)
    assert OmegaConf.create(str(result.decode("utf-8"))) == output_conf


@pytest.mark.parametrize(
    "args,output_conf",
    [
        (
            [],
            OmegaConf.create(
                {
                    "optimizer": {"lr": 0.001, "type": "nesterov"},
                    "dataset": {"name": "imagenet", "path": "/datasets/imagenet"},
                }
            ),
        ),
        (
            ["optimizer=adam"],
            OmegaConf.create(
                {
                    "optimizer": {"beta": 0.01, "lr": 0.1, "type": "adam"},
                    "dataset": {"name": "imagenet", "path": "/datasets/imagenet"},
                }
            ),
        ),
    ],
)
def test_demo_5_config_groups(tmpdir, args, output_conf):
    cmd = [
        sys.executable,
        "demos/5_config_groups/defaults.py",
        "hydra.run.dir=" + str(tmpdir),
    ]
    cmd.extend(args)
    result = subprocess.check_output(cmd)
    assert OmegaConf.create(str(result.decode("utf-8"))) == output_conf


def test_fairtask_sweep_1_job(sweep_runner):  # noqa: F811
    sweep_1_job(sweep_runner, overrides=["hydra/launcher=basic"])


def test_fairtask_sweep_2_jobs(sweep_runner):  # noqa: F811
    sweep_2_jobs(sweep_runner, overrides=["hydra/launcher=basic"])


def test_not_sweeping_hydra_overrides(sweep_runner):  # noqa: F811
    not_sweeping_hydra_overrides(sweep_runner, overrides=["hydra/launcher=basic"])


def test_fairtask_sweep_1_job_strict(sweep_runner):  # noqa: F811
    sweep_1_job(sweep_runner, strict=True, overrides=["hydra/launcher=basic"])


def test_fairtask_sweep_1_job_strict_and_bad_key(sweep_runner):  # noqa: F811
    with pytest.raises(KeyError):
        sweep_1_job(
            sweep_runner,
            strict=True,
            overrides=["hydra/launcher=basic", "hydra.foo=bar"],
        )


def test_fairtask_sweep_2_optimizers(sweep_runner):  # noqa: F811
    sweep_two_config_groups(sweep_runner, overrides=["hydra/launcher=basic"])


def test_specializing_configs_demo(task_runner):  # noqa: F811
    with task_runner(
        calling_file="demos/8_specializing_config/example.py",
        calling_module=None,
        config_path="conf/config.yaml",
        overrides=["dataset=cifar10"],
    ) as task:
        assert task.job_ret.cfg == dict(
            dataset=dict(name="cifar10", path="/datasets/cifar10"),
            model=dict(num_layers=5, type="alexnet"),
        )
        verify_dir_outputs(task.job_ret, overrides=task.overrides)


def test_short_module_name():
    try:
        os.chdir("demos/3_config_file")
        cmd = [sys.executable, "config_file.py"]
        modified_env = os.environ.copy()
        modified_env["HYDRA_MAIN_MODULE"] = "config_file"
        result = subprocess.check_output(cmd, env=modified_env)
        assert OmegaConf.create(str(result.decode("utf-8"))) == dict(
            dataset=dict(name="imagenet", path="/datasets/imagenet")
        )
    finally:
        chdir_hydra_root()


def test_objects_demo(task_runner):  # noqa: F811
    with task_runner(
        calling_file="demos/7_objects/experiment.py",
        calling_module=None,
        config_path="conf/config.yaml",
        overrides=[],
    ) as task:
        assert task.job_ret.cfg == {
            "model": {
                "class": "demos.7_objects.objects.Alexnet",
                "params": {"num_layers": 7},
            }
        }
        verify_dir_outputs(task.job_ret, overrides=task.overrides)



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
import importlib

import pytest
from omegaconf import OmegaConf

from hydra.test_utils.test_utils import integration_test

# noinspection PyUnresolvedReferences
from hydra.test_utils.test_utils import sweep_runner  # noqa: F401


def verify_plugin(plugin_module):
    if plugin_module is not None:
        try:
            importlib.import_module(plugin_module)
        except ImportError:
            pytest.skip("Plugin {} not installed".format(plugin_module))


def create_basic_launcher_config():
    return OmegaConf.create(
        {
            "defaults": [
                {"hydra/hydra_logging": "hydra_debug"},
                {"hydra/job_logging": "disabled"},
                {"hydra/launcher": "basic"},
            ]
        }
    )


def create_fairtask_launcher_local_config():
    return OmegaConf.create(
        {
            "defaults": [
                {"hydra/launcher": None},
                {"hydra/hydra_logging": "hydra_debug"},
                {"hydra/job_logging": "disabled"},
            ],
            "hydra": {
                "launcher": {
                    "class": "hydra_plugins.fairtask.FAIRTaskLauncher",
                    "params": {
                        "no_workers": True,
                        "queue": "local",
                        "queues": {
                            "local": {
                                "class": "fairtask.local.LocalQueueConfig",
                                "params": {"num_workers": 2},
                            }
                        },
                    },
                }
            },
        }
    )


def create_submitit_launcher_local_config():
    return OmegaConf.create(
        {
            "defaults": [
                {"hydra/launcher": None},
                {"hydra/hydra_logging": "hydra_debug"},
                {"hydra/job_logging": "disabled"},
            ],
            "hydra": {
                "launcher": {
                    "class": "hydra_plugins.submitit.SubmititLauncher",
                    "params": {
                        "queue": "local",
                        "folder": "${hydra.sweep.dir}/.${hydra.launcher.params.queue}",
                        "queue_parameters": {
                            "local": {
                                "gpus_per_node": 1,
                                "tasks_per_node": 1,
                                "timeout_min": 1,
                            }
                        },
                    },
                }
            },
        }
    )


@pytest.mark.parametrize(
    "task_config, overrides, filename, expected_name",
    [
        (None, [], "no_config.py", "no_config"),
        (None, ["hydra.job.name=overridden_name"], "no_config.py", "overridden_name"),
        (
            OmegaConf.create({"hydra": {"job": {"name": "name_from_config_file"}}}),
            [],
            "with_config.py",
            "name_from_config_file",
        ),
        (
            OmegaConf.create(dict(hydra=dict(name="name_from_config_file"))),
            ["hydra.job.name=overridden_name"],
            "with_config.py",
            "overridden_name",
        ),
    ],
)
@pytest.mark.parametrize(
    "hydra_config, extra_flags, plugin_module",
    [
        (create_basic_launcher_config(), ["-m", "hydra.sweep.dir=."], None),
        (
            create_fairtask_launcher_local_config(),
            ["-m", "hydra.sweep.dir=."],
            "hydra_plugins.fairtask",
        ),
        # # TODO: re-enable after submitit local queue is fixed
        # pytest.param(
        #     create_submitit_launcher_local_config(), ['-m'], 'hydra_plugins.submitit',
        #     marks=[pytest.mark.skip]
        # ),
    ],
)
def test_custom_task_name(
    tmpdir,
    task_config,
    overrides,
    filename,
    expected_name,
    hydra_config,
    extra_flags,
    plugin_module,
):
    verify_plugin(plugin_module)
    overrides = extra_flags + overrides
    cfg = OmegaConf.merge(
        hydra_config or OmegaConf.create(), task_config or OmegaConf.create()
    )
    integration_test(
        tmpdir=tmpdir,
        task_config=cfg,
        overrides=overrides,
        prints="HydraConfig().hydra.job.name",
        expected_outputs=expected_name,
        filename=filename,
    )


@pytest.mark.parametrize(
    "task_config, overrides, expected_dir",
    [
        ({"hydra": {"run": {"dir": "foo"}}}, [], "foo"),
        ({}, ["hydra.run.dir=bar"], "bar"),
        ({"hydra": {"run": {"dir": "foo"}}}, ["hydra.run.dir=boom"], "boom"),
        (
            {"hydra": {"run": {"dir": "foo-${hydra.job.override_dirname}"}}},
            ["a=1", "b=2"],
            "foo-a=1,b=2",
        ),
    ],
)
def test_custom_local_run_workdir(tmpdir, task_config, overrides, expected_dir):
    cfg = OmegaConf.create(task_config)
    expected_dir1 = tmpdir / expected_dir
    integration_test(
        tmpdir=tmpdir,
        task_config=cfg,
        overrides=overrides,
        prints="os.getcwd()",
        expected_outputs=str(expected_dir1),
    )


@pytest.mark.parametrize(
    "task_config, overrides, expected_dir",
    [
        (
            {
                "hydra": {
                    "sweep": {"dir": "task_cfg", "subdir": "task_cfg_${hydra.job.num}"}
                }
            },
            [],
            "task_cfg/task_cfg_0",
        ),
        (
            {},
            ["hydra.sweep.dir=cli_dir", "hydra.sweep.subdir=cli_dir_${hydra.job.num}"],
            "cli_dir/cli_dir_0",
        ),
        (
            {
                "hydra": {
                    "sweep": {"dir": "task_cfg", "subdir": "task_cfg_${hydra.job.num}"}
                }
            },
            ["hydra.sweep.dir=cli_dir", "hydra.sweep.subdir=cli_dir_${hydra.job.num}"],
            "cli_dir/cli_dir_0",
        ),
        (
            {
                "hydra": {
                    "sweep": {
                        "dir": "hydra_cfg",
                        "subdir": "${hydra.job.override_dirname}",
                    }
                }
            },
            ["a=1", "b=2"],
            "hydra_cfg/a=1,b=2",
        ),
    ],
)
@pytest.mark.parametrize(
    "task_launcher_cfg, extra_flags, plugin_module",
    [
        (create_basic_launcher_config(), ["-m"], None),
        (create_fairtask_launcher_local_config(), ["-m"], "hydra_plugins.fairtask"),
        # TODO: re-enable after submitit local queue is fixed
        pytest.param(
            create_submitit_launcher_local_config(),
            ["-m"],
            "hydra_plugins.submitit",
            marks=[pytest.mark.skip],
        ),
    ],
)
def test_custom_sweeper_run_workdir(
    tmpdir,
    task_config,
    overrides,
    expected_dir,
    task_launcher_cfg,
    extra_flags,
    plugin_module,
):
    verify_plugin(plugin_module)
    if task_config is not None:
        task_config = OmegaConf.create(task_config)

    overrides = extra_flags + overrides
    cfg = OmegaConf.merge(task_launcher_cfg, task_config)

    integration_test(
        tmpdir=tmpdir,
        task_config=cfg,
        overrides=overrides,
        prints="os.getcwd()",
        expected_outputs=str(tmpdir / expected_dir),
    )



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
import pytest

from hydra.test_utils.launcher_common_tests import LauncherTestSuite
from hydra.test_utils.test_utils import chdir_hydra_root

chdir_hydra_root()

# noinspection PyUnresolvedReferences
from hydra.test_utils.test_utils import sweep_runner  # noqa: F401


@pytest.mark.parametrize("launcher_name, overrides", [("basic", [])])
class TestBasicLauncher(LauncherTestSuite):
    pass



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
import logging
import os
from os.path import realpath, dirname, splitext, basename

from omegaconf import open_dict
from collections import defaultdict
from .config_loader import ConfigLoader
from .config_search_path import ConfigSearchPath
from .plugins import Plugins
from ..errors import MissingConfigException
from ..plugins import SearchPathPlugin, Launcher, Sweeper, CompletionPlugin
from ..plugins.common.utils import (
    configure_log,
    run_job,
    get_valid_filename,
    JobRuntime,
    HydraConfig,
    setup_globals,
    get_overrides_dirname,
)

log = None


class Hydra:
    def __init__(
        self, calling_file, calling_module, config_path, task_function, verbose, strict
    ):
        setup_globals()
        assert calling_module is not None or calling_file is not None
        basedir_prefix = ""
        if calling_module is None:
            # executed with python file.py
            abs_base_dir = realpath(dirname(calling_file))
            target_file = os.path.basename(calling_file)
            task_name = os.path.splitext(target_file)[0]
        else:
            # module is installed, use pkg:// access to get configs
            last_dot = calling_module.rfind(".")
            if last_dot != -1:
                task_name = calling_module[last_dot + 1 :]
            else:
                task_name = calling_module
            basedir_prefix = "pkg://"
            abs_base_dir = calling_module

        JobRuntime().set("name", get_valid_filename(task_name))
        self.task_name = task_name
        self.task_function = task_function

        split_file = splitext(config_path)
        if split_file[1] in (".yaml", ".yml"):
            # assuming dir/config.yaml form
            config_file = basename(config_path)
            config_dir = dirname(config_path)
        else:
            # assuming dir form without a config file.
            config_file = None
            config_dir = config_path

        if config_dir != "":
            abs_config_dir = "{}/{}".format(abs_base_dir, config_dir)
        else:
            abs_config_dir = abs_base_dir

        abs_config_dir = basedir_prefix + abs_config_dir

        search_path = ConfigSearchPath()
        search_path.append("hydra", "pkg://hydra.conf")
        search_path.append("main", abs_config_dir)

        search_path_plugins = Plugins.discover(SearchPathPlugin)
        for spp in search_path_plugins:
            plugin = spp()
            plugin.manipulate_search_path(search_path)

        self.config_loader = ConfigLoader(
            config_file=config_file, config_search_path=search_path, strict_cfg=strict
        )
        if not self.config_loader.exists(abs_config_dir):
            raise MissingConfigException(
                missing_cfg_file=abs_config_dir,
                message="Primary config dir not found: {}".format(abs_config_dir),
            )

        if config_file is not None and not self.config_loader.exists_in_search_path(
            config_file
        ):
            raise MissingConfigException(
                missing_cfg_file=config_file,
                message="Cannot find primary config file: {}".format(config_file),
            )
        self.verbose = verbose

    def run(self, overrides):
        cfg = self._load_config(overrides)
        HydraConfig().set_config(cfg)
        with open_dict(cfg):
            cfg.hydra.job.override_dirname = get_overrides_dirname(
                cfg.hydra.overrides.task
            )

        return run_job(
            config=cfg,
            task_function=self.task_function,
            verbose=self.verbose,
            job_dir_key="hydra.run.dir",
            job_subdir_key=None,
        )

    def multirun(self, overrides):
        cfg = self._load_config(overrides)
        HydraConfig().set_config(cfg)
        sweeper = Plugins.instantiate_sweeper(
            config=cfg,
            config_loader=self.config_loader,
            task_function=self.task_function,
            verbose=self.verbose,
        )
        task_overrides = cfg.hydra.overrides.task
        return sweeper.sweep(arguments=task_overrides)

    def show_cfg(self, overrides):
        config = self._load_config(overrides)
        log.info("\n" + config.pretty())

    @staticmethod
    def get_shell_to_plugin_map(config_loader):
        shell_to_plugin = defaultdict(list)
        for clazz in Plugins.discover(CompletionPlugin):
            plugin = clazz(config_loader)
            shell_to_plugin[plugin.provides()].append(plugin)

        for shell, plugins in shell_to_plugin.items():
            if len(plugins) > 1:
                raise ValueError(
                    "Multiple plugins installed for {} : {}".format(
                        shell, ",".join([type(plugin).__name__ for plugin in plugins])
                    )
                )

        return shell_to_plugin

    def shell_completion(self, overrides):
        config = self._load_config(overrides)
        subcommands = ["install", "uninstall", "query"]
        found = False
        for sc in subcommands:
            if sc in config:
                found = True
                break
        if not found:
            log.error(
                "No completion subcommand specified ({})".format(",".join(subcommands))
            )

        shell_to_plugin = self.get_shell_to_plugin_map(self.config_loader)

        def find_plugin(cmd):
            if cmd not in shell_to_plugin:
                raise ValueError(
                    "No completion plugin for '{}' found, available : \n{}".format(
                        cmd, "\n".join(["\t" + x for x in shell_to_plugin.keys()])
                    )
                )
            return shell_to_plugin[cmd][0]

        if config.install:
            plugin = find_plugin(config.install)
            plugin.install()
        elif config.uninstall:
            plugin = find_plugin(config.uninstall)
            plugin.uninstall()
        elif config.query:
            plugin = find_plugin(config.query)
            plugin.query()

    @staticmethod
    def _log_header(header, prefix="", filler="-"):
        log.debug(prefix + header)
        log.debug(prefix + "".ljust(len(header), filler))

    def _print_plugins(self):
        self._log_header(header="Installed Hydra Plugins", filler="*")
        for plugin_type in [SearchPathPlugin, Sweeper, Launcher]:
            Hydra._log_header(
                header="{}:".format(plugin_type.__name__), prefix="\t", filler="-"
            )
            for plugin in Plugins.discover(plugin_type):
                log.debug("\t\t{}".format(plugin.__name__))

    def _get_padding(self):
        provider_pad = 0
        search_path_pad = 0
        file_pad = 0
        for sp in self.config_loader.config_search_path.config_search_path:
            provider_pad = max(provider_pad, len(sp.provider))
            search_path_pad = max(search_path_pad, len(sp.path))
        for file, _, _ in self.config_loader.get_load_history():
            file_pad = max(file_pad, len(file))

        provider_pad += 1
        search_path_pad += 1
        file_pad += 1
        return provider_pad, search_path_pad, file_pad

    def _print_search_path(self):
        log.debug("")
        self._log_header(header="Hydra config search path", filler="*")

        provider_pad, search_path_pad, file_pad = self._get_padding()
        self._log_header(
            "| {} | {} |".format(
                "Provider".ljust(provider_pad), "Search path".ljust(search_path_pad)
            ),
            filler="-",
        )

        for sp in self.config_loader.config_search_path.config_search_path:
            log.debug(
                "| {} | {} |".format(
                    sp.provider.ljust(provider_pad), sp.path.ljust(search_path_pad)
                )
            )

    def _print_composition_trace(self):
        # Print configurations used to compose the config object
        provider_pad, search_path_pad, file_pad = self._get_padding()
        log.debug("")
        self._log_header("Composition trace", filler="*")
        self._log_header(
            "| {} | {} | {} |".format(
                "Provider".ljust(provider_pad),
                "Search path".ljust(search_path_pad),
                "File".ljust(file_pad),
            ),
            filler="-",
        )

        for file, search_path, provider in self.config_loader.get_load_history():
            if search_path is not None:
                log.debug(
                    "| {} | {} | {} |".format(
                        provider.ljust(provider_pad),
                        search_path.ljust(search_path_pad),
                        file.ljust(file_pad),
                    )
                )
            else:
                log.debug("{} : NOT FOUND".format(file))

    def _print_debug_info(self):
        self._print_plugins()
        self._print_search_path()
        self._print_composition_trace()

    def _load_config(self, overrides):
        cfg = self.config_loader.load_configuration(overrides)
        configure_log(cfg.hydra.hydra_logging, self.verbose)
        global log
        log = logging.getLogger(__name__)
        self._print_debug_info()
        return cfg



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved

import pytest
from omegaconf import OmegaConf

from hydra.test_utils.test_utils import create_search_path
from hydra._internal.config_loader import ConfigLoader
from hydra.errors import MissingConfigException
from hydra.test_utils.test_utils import chdir_hydra_root
import pkg_resources

chdir_hydra_root()


def test_override_run_dir_without_hydra_cfg():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(), strict_cfg=False, config_file=None
    )
    cfg = config_loader.load_configuration(overrides=["hydra.run.dir=abc"])
    assert cfg.hydra.run.dir == "abc"


def test_override_run_dir_with_hydra_cfg():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(
            ["demos/99_hydra_configuration/workdir/"]
        ),
        strict_cfg=False,
        config_file=None,
    )
    cfg = config_loader.load_configuration(overrides=["hydra.run.dir=abc"])
    assert cfg.hydra.run.dir == "abc"


def test_load_configuration():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["demos/3_config_file"]),
        strict_cfg=False,
        config_file="config.yaml",
    )
    cfg = config_loader.load_configuration(overrides=["abc=123"])
    del cfg["hydra"]
    assert cfg == OmegaConf.create(
        dict(abc=123, dataset=dict(name="imagenet", path="/datasets/imagenet"))
    )


def test_load_with_missing_default():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="missing-default.yaml",
    )
    with pytest.raises(MissingConfigException):
        config_loader.load_configuration()


def test_load_with_missing_optional_default():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="missing-optional-default.yaml",
    )
    cfg = config_loader.load_configuration()
    del cfg["hydra"]
    assert cfg == {}


def test_load_with_optional_default():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="optional-default.yaml",
    )
    cfg = config_loader.load_configuration()
    del cfg["hydra"]
    assert cfg == dict(foo=10)


def test_load_changing_group_in_default():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="optional-default.yaml",
    )
    cfg = config_loader.load_configuration(["group1=file2"])
    del cfg["hydra"]
    assert cfg == dict(foo=20)


def test_load_adding_group_not_in_default():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="optional-default.yaml",
    )
    cfg = config_loader.load_configuration(["group2=file1"])
    del cfg["hydra"]
    assert cfg == dict(foo=10, bar=100)


def test_load_history():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="missing-optional-default.yaml",
    )
    config_loader.load_configuration()
    assert config_loader.get_load_history() == [
        ("hydra.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/hydra_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/job_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/launcher/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/sweeper/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/output/default.yaml", "pkg://hydra.conf", "hydra"),
        ("missing-optional-default.yaml", "hydra/test_utils/configs", "test"),
        ("foo/missing.yaml", None, None),
    ]


def test_load_history_with_basic_launcher():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["demos/6_sweep/conf"]),
        strict_cfg=False,
        config_file="config.yaml",
    )
    config_loader.load_configuration(overrides=["hydra/launcher=basic"])

    assert config_loader.get_load_history() == [
        ("hydra.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/hydra_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/job_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/launcher/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/sweeper/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/output/default.yaml", "pkg://hydra.conf", "hydra"),
        ("config.yaml", "demos/6_sweep/conf", "test"),
        ("optimizer/nesterov.yaml", "demos/6_sweep/conf", "test"),
    ]


def test_load_strict():
    """
    Ensure that in strict mode we can override only things that are already in the config
    :return:
    """
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["demos/3_config_file"]),
        config_file="config.yaml",
        strict_cfg=True,
    )
    cfg = config_loader.load_configuration(overrides=["dataset.name=foobar"])
    del cfg["hydra"]
    assert cfg == dict(dataset=dict(name="foobar", path="/datasets/imagenet"))
    with pytest.raises(KeyError):
        cfg.not_here

    with pytest.raises(KeyError):
        cfg.dataset.not_here

    config_loader = ConfigLoader(
        config_search_path=create_search_path(["demos/3_config_file"]),
        config_file="config.yaml",
        strict_cfg=True,
    )
    with pytest.raises(KeyError):
        config_loader.load_configuration(overrides=["dataset.bad_key=foobar"])


def test_load_yml_file():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="config.yml",
    )
    cfg = config_loader.load_configuration()
    del cfg["hydra"]
    assert cfg == dict(yml_file_here=True)


@pytest.mark.parametrize(
    "primary,merged,result",
    [
        ([], [], []),
        ([{"a": 10}], [], [{"a": 10}]),
        ([{"a": 10}, {"b": 20}], [{"a": 20}], [{"a": 20}, {"b": 20}]),
        (
            [
                {"hydra_logging": "default"},
                {"job_logging": "default"},
                {"launcher": "basic"},
                {"sweeper": "basic"},
            ],
            [{"optimizer": "nesterov"}, {"launcher": "basic"}],
            [
                {"hydra_logging": "default"},
                {"job_logging": "default"},
                {"launcher": "basic"},
                {"sweeper": "basic"},
                {"optimizer": "nesterov"},
            ],
        ),
    ],
)
def test_merge_default_lists(primary, merged, result):
    ConfigLoader._merge_default_lists(primary, merged)
    assert primary == result


@pytest.mark.parametrize(
    "config_file, overrides",
    [
        # remove from config
        ("removing-hydra-launcher-default.yaml", []),
        # remove from override
        ("config.yaml", ["hydra/launcher=null"]),
        # remove from both
        ("removing-hydra-launcher-default.yaml", ["hydra/launcher=null"]),
        # second overrides removes
        ("config.yaml", ["hydra/launcher=submitit", "hydra/launcher=null"]),
    ],
)
def test_default_removal(config_file, overrides):
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file=config_file,
    )
    config_loader.load_configuration(overrides=overrides)
    assert config_loader.get_load_history() == [
        ("hydra.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/hydra_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/job_logging/default.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/sweeper/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/output/default.yaml", "pkg://hydra.conf", "hydra"),
        (config_file, "hydra/test_utils/configs", "test"),
    ]


def test_defaults_not_list_exception():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="defaults_not_list.yaml",
    )
    with pytest.raises(ValueError):
        config_loader.load_configuration(overrides=[])


@pytest.mark.parametrize(
    "module_name, resource_name",
    [
        ("hydra.test_utils", ""),
        ("hydra.test_utils", "__init__.py"),
        ("hydra.test_utils", "configs"),
        ("hydra.test_utils", "configs/config.yaml"),
        ("hydra.test_utils.configs", ""),
        ("hydra.test_utils.configs", "config.yaml"),
        ("hydra.test_utils.configs", "group1"),
        ("hydra.test_utils.configs", "group1/file1.yaml"),
    ],
)
def test_resource_exists(module_name, resource_name):
    assert pkg_resources.resource_exists(module_name, resource_name) is True


def test_override_hydra_config_value_from_config_file():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="overriding_output_dir.yaml",
    )

    cfg = config_loader.load_configuration(overrides=[])
    assert cfg.hydra.run.dir == "foo"


def test_override_hydra_config_group_from_config_file():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="overriding_logging_default.yaml",
    )

    config_loader.load_configuration(overrides=[])
    assert config_loader.get_load_history() == [
        ("hydra.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/hydra_logging/hydra_debug.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/job_logging/disabled.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/sweeper/basic.yaml", "pkg://hydra.conf", "hydra"),
        ("hydra/output/default.yaml", "pkg://hydra.conf", "hydra"),
        ("overriding_logging_default.yaml", "hydra/test_utils/configs", "test"),
    ]


def test_list_groups():
    config_loader = ConfigLoader(
        config_search_path=create_search_path(["hydra/test_utils/configs"]),
        strict_cfg=False,
        config_file="overriding_output_dir.yaml",
    )
    assert sorted(config_loader.list_groups("")) == [
        "completion_test",
        "group1",
        "group2",
        "hydra",
    ]

    assert sorted(config_loader.list_groups("hydra")) == [
        "hydra_logging",
        "job_logging",
        "launcher",
        "output",
        "sweeper",
    ]



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
import inspect
import os
import sys
from .hydra import Hydra
import argparse
from argparse import RawTextHelpFormatter


def run_hydra(args, task_function, config_path, strict):
    stack = inspect.stack()
    frame = stack[2]

    calling_file = None
    calling__module = None
    try:
        calling_file = frame[0].f_locals["__file__"]
    except KeyError:
        pass
    try:
        module_envs = ["HYDRA_MAIN_MODULE", "FB_PAR_MAIN_MODULE", "FB_XAR_MAIN_MODULE"]
        for module_env in module_envs:
            if module_env in os.environ:
                calling__module = os.environ[module_env]
                break

        if calling__module is None:
            calling__module = frame[0].f_globals[frame[3]].__module__
    except KeyError:
        pass

    hydra = Hydra(
        calling_file=calling_file,
        calling_module=calling__module,
        config_path=config_path,
        task_function=task_function,
        verbose=args.verbose,
        strict=strict,
    )

    num_commands = args.run + args.cfg + args.multirun + args.shell_completion
    if num_commands > 1:
        raise ValueError(
            "Only one of --run, --sweep and --cfg  --completion can be specified"
        )
    if num_commands == 0:
        args.run = True

    if args.run:
        hydra.run(overrides=args.overrides)
    elif args.multirun:
        hydra.multirun(overrides=args.overrides)
    elif args.cfg:
        hydra.show_cfg(overrides=args.overrides)
    elif args.shell_completion:
        hydra.shell_completion(overrides=args.overrides)
    else:
        print("Command not specified")
        sys.exit(1)


def _get_exec_command():
    if sys.argv[0].endswith(".py"):
        return "python {}".format(sys.argv[0])
    else:
        # Running as an installed app (setuptools entry point)
        executable = os.path.basename(sys.argv[0])
        return executable


def get_args(args=None, version=None):
    parser = argparse.ArgumentParser(
        description="Hydra", formatter_class=RawTextHelpFormatter
    )
    if version is not None:
        parser.add_argument(
            "--version", action="version", version="Hydra {}".format(version)
        )
    parser.add_argument(
        "overrides",
        nargs="*",
        help="""Any key=value arguments to override config values
(use dots for.nested=overrides)""",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        help="""Activate debug logging, otherwise takes a comma"
separated list of loggers ('root' for root logger)""",
        nargs="?",
        default=None,
    )

    parser.add_argument("--cfg", "-c", action="store_true", help="Show config")

    parser.add_argument("--run", "-r", action="store_true", help="Run a job")

    parser.add_argument(
        "--multirun",
        "-m",
        action="store_true",
        help="Run multiple jobs with the configured launcher",
    )

    shell = "SHELL_NAME"
    install_cmd = 'eval "$({} -sc install={})"'.format(_get_exec_command(), shell)
    uninstall_cmd = 'eval "$({} --sc uninstall={})"'.format(_get_exec_command(), shell)
    parser.add_argument(
        "--shell_completion",
        "-sc",
        action="store_true",
        help="""Install or Uninstall shell completion:
Install:
{}

Uninstall:
{}
""".format(
            install_cmd, uninstall_cmd
        ),
    )

    return parser.parse_args(args)
